/* sottografo.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "grafo-ma.h"


#define ROW_LENGTH 256


/* Prototipi delle procedure secondarie e definizione di tipi */

typedef int *vint;

/* Legge da linea di comando il nome del file che contiene il grafo
   e gli indici dei nodi del sottografo */
void InterpretaLineaComando (int argc, char *argv[], char *filedati, vint *pS, int *pns);

/* Carica il grafo G dal file filedati */
void CaricaGrafo (char *filedati, grafo *pG);

/* Stampa a video il sottografo indotto sul grafo G dal sottoinsieme S di ns nodi */
void StampaSottografoIndotto (grafo *pG, vint S, int ns);


/* Programma principale */
int main (int argc, char *argv[])
{
  char filedati[ROW_LENGTH];
  vint S;
  int ns;
  grafo G;


  /* Legge da linea di comando il nome del file che contiene il grafo
     e gli indici dei nodi del sottografo */
  InterpretaLineaComando(argc,argv,filedati,&S,&ns);

  /* Carica il grafo da file */
  CaricaGrafo(filedati,&G);

  /* Stampa a video il grafo */
  stampagrafo(&G);

  /* Stampa a video il sottografo indotto */
  StampaSottografoIndotto(&G,S,ns);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

/* Legge da linea di comando il nome del file che contiene il grafo
   e gli indici dei nodi del sottografo */
void InterpretaLineaComando (int argc, char *argv[], char *filedati, vint *pS, int *pns)
{
  int p;

  /* Controlla il numero degli argomenti */
  if (argc < 2)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    exit(EXIT_FAILURE);
  }

  /* Determina il nome del file dei dati */
  strcpy(filedati,argv[1]);

  /* Determina il numero di nodi del sottografo indotto */
  *pns = argc-2;

  /* Alloca il vettore dei nodi del sottografo indotto */
  *pS = (vint) calloc(*pns+1,sizeof(int));
  if (*pS == NULL)
  {
    fprintf(stderr,"Errore nell'allocazione del vettore dei nodi del sottografo indotto!\n");
    exit(EXIT_FAILURE);
  }

  /* Riempie il vettore dei nodi del sottografo indotto */
  for (p = 1; p <= *pns; p++)
    if (sscanf(argv[p+1],"%d",&(*pS)[p]) != 1)
    {
      fprintf(stderr,"Errore nel formato della linea di comando!\n");
      exit(EXIT_FAILURE);
    }
}


/* Carica il grafo G dal file filedati */
void CaricaGrafo (char *filedati, grafo *pG)
{
  FILE *fp;
  int n;
  int o, d;


  /* Apre il file */
  fp = fopen(filedati,"r");
  if (fp == NULL)
  {
    fprintf(stderr,"Errore nell'apertura del file %s!\n",filedati);
    exit(EXIT_FAILURE);
  }

  /* Conta i nodi del grafo: legge gli archi e salva il massimo indice */
  n = 0;
  while (fscanf(fp," (%d ,%d )",&o,&d) == 2)
  {
    if (o > n) n = o;
    if (d > n) n = d;
  }

  /* Torna all'inizio del file */
  rewind(fp);

  /* Crea un grafo vuoto di n nodi */
  creagrafo(n,pG);

  /* Scorre gli archi inserendoli nel grafo */
  while (fscanf(fp," (%d ,%d )",&o,&d) == 2)
    insarco(o,d,pG);

  /* Chiude il file */
  fclose(fp);
}


/* Stampa a video il sottografo indotto sul grafo G dal sottoinsieme S di ns nodi */
void StampaSottografoIndotto (grafo *pG, vint S, int ns)
{
}
